// some useful routines for writing 3D interactive programs
// written for CS638 - Michael Gleicher, November 1999

#ifdef _LINUX
#else
	#include <windows.h>
#endif

#include "3DUtils.h"
#include "quaternion.h"
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>

#ifndef PI
#define PI (3.1415926536)
#endif

//Computes the cross product of two 3-vec's and returns the value (by reference) as a third
//  3-vec.
void crossProduct(float ax, float ay, float az, float bx, float by, float bz, float &cx, float &cy, float &cz)
{
	cx = ay * bz - az * by;
	cy = az * bx - ax * bz;
	cz = ax * by - ay * bx;
}

// This function takes two points and draws an ellipse between them
void ellipsify(float x1, float y1, float z1, float x2, float y2, float z2)
{
	glPushMatrix()   ;

	float distance = (float)(sqrt (((x1 - x2)*(x1-x2)) + ((y1 - y2)*(y1-y2)) + ((z1 - z2)*(z1-z2)))) ;
	float distanceXZ = (float)(sqrt ((x2-x1)*(x2-x1)+(z2-z1)*(z2-z1)));

	// Move to the first point on the ellipse
	glTranslatef(x1,y1,z1);
	

	// Here we orient the x-axis so that it is in the XZ plane pointing int the direction 
	// we want the ellipse to go
	float rotY = (float)(atan((z2-z1)/(x2-x1))*180/PI);
	if( (x2-x1)<0 )
		glRotatef( -180.0f-rotY, 0, 1, 0 );
	else if( (x2-x1)>0 )	
		glRotatef( -rotY, 0, 1, 0 );
	else if( (z2-z1) > 0 )
		glRotatef( -90, 0, 1, 0 );
	else
		glRotatef( 90, 0, 1, 0 );

	
	// Now we want to rotate about Z so that the x-axis rotates up and is now pointing
	// down the ellipsoid
	float rotZ = (float)(atan( (y2-y1)/distanceXZ )*180/PI);
	if( distanceXZ != 0)
		glRotatef( rotZ, 0, 0, 1 );
	else
		glRotatef( 90, 0, 0, 1 );


	// X-axis should now be pointing down direction that we want our ellipsoid to go
	// So we now scale the axis so that when we draw a circle we get an ellipse
	// We then translate halfway down the x-axis and draw our circle which turns out
	// to be an ellipse between the two points given
	glScalef (1.0f, 0.35f, 0.35f) ;
	glTranslatef(distance/2.0f,0,0);
	GLUquadricObj* qSphere = gluNewQuadric() ;
	gluSphere (qSphere, (distance/2.0f), 30, 30)  ;

	//How does this work?
	//dropShadowMatrix() ;
	//gluSphere (qSphere, (distance/2.0f), 30, 30)  ;

	glPopMatrix() ;
}


//This function returns the singular value of two 3-vec's dot products...
float dotProduct(float x1, float y1, float z1, float x2, float y2, float z2) 
{
	return ((x1*x2) + (y1*y2) + (z1*z2)) ;
}


// draw a cube
// note: this isn't necesarily the fastest way since I recompute each
// vertex
// also, notice that I don't keep all my polygons with the same orientations!
void drawCube(double x, double y, double z, double l)
{
	// we'll use a transform to move the cube to the right place, however
	// we won't use it for the scale since that might mess up lighting
	glPushMatrix();
	glTranslated(x,y,z);
	glBegin(GL_QUADS);
	glNormal3d( 0,0,1);
	glVertex3d( l/2, l/2, l/2);
	glVertex3d(-l/2, l/2, l/2);
	glVertex3d(-l/2,-l/2, l/2);
	glVertex3d( l/2,-l/2, l/2);
	glNormal3d( 0,0,-1);
	glVertex3d( l/2, l/2, -l/2);
	glVertex3d(-l/2, l/2, -l/2);
	glVertex3d(-l/2,-l/2, -l/2);
	glVertex3d( l/2,-l/2, -l/2);
	glNormal3d( 0,1,0);
	glVertex3d( l/2, l/2, l/2);
	glVertex3d(-l/2, l/2, l/2);
	glVertex3d(-l/2, l/2,-l/2);
	glVertex3d( l/2, l/2,-l/2);
	glNormal3d( 0,-1,0);
	glVertex3d( l/2,-l/2, l/2);
	glVertex3d(-l/2,-l/2, l/2);
	glVertex3d(-l/2,-l/2,-l/2);
	glVertex3d( l/2,-l/2,-l/2);
	glNormal3d( 1,0,0);
	glVertex3d( l/2, l/2, l/2);
	glVertex3d( l/2,-l/2, l/2);
	glVertex3d( l/2,-l/2,-l/2);
	glVertex3d( l/2, l/2,-l/2);
	glNormal3d(-1,0,0);
	glVertex3d(-l/2, l/2, l/2);
	glVertex3d(-l/2,-l/2, l/2);
	glVertex3d(-l/2,-l/2,-l/2);
	glVertex3d(-l/2, l/2,-l/2);
	glEnd();
	glPopMatrix();
}


//This is a modified version of the code provided by Dr. Michael Gleicher.
//  It simply draws the top face of the cube, rather than the whole cube.
//  This is good for drawing boxes in orthogonal views.
void drawBox(double x, double y, double z, double w, double h)
{
	// we'll use a transform to move the cube to the right place, however
	// we won't use it for the scale since that might mess up lighting
	glPushMatrix();
	glTranslated(x,y,z);
	glBegin(GL_QUADS);
	glNormal3d( 0,0,1);
	glVertex3d( w/2, h/2, 0);
	glVertex3d(-w/2, h/2, 0);
	glVertex3d(-w/2,-h/2, 0);
	glVertex3d( w/2,-h/2, 0);
	glEnd();
	glPopMatrix();
}

// Draw a line between the points that get passed in
void drawLine(double x1, double y1, double z1, double x2, double y2, double z2)
{
	glPushMatrix();
	glBegin(GL_LINES);
	glVertex3d(x1, y1, z1);
	glVertex3d(x2, y2, z2);
	glEnd();
	glPopMatrix();
}

// a shadow matrix multiplied onto the stack
// notice: we smash the y value to near 0, but not quite
// the shadows actually float slightly above the floor
// the is a really crummy hack, but it avoids problems with the Z-buffer
// there are better ways to do this
// the matrix is:
//	1  0  0  0
//  0  0  0  .05	--- .05 is the float
//  0  0  1  0
//  0  0  0  1
double dsm[16] = {1,0,0,0, 0,0,0,0, 0,0,1,0, 0,.05,0,1};
void dropShadowMatrix( float floorHeight )
{
	dsm[13] = floorHeight+.05;
	glMultMatrixd(dsm);
}

// This function computes the cardinal-ROM spline
// If you pass in 4 points p0, p1, p2, and p3 it will return a point
// on the curve between points p1 and p3.  The u values should be between
// 0 and 1 and it specifies where along the curve we are, a value of 1
// will be at p2 and a value of 0 will be at p1
double cardinal( double u, double p0, double p1, double p2, double p3, double tension )
{
	double s = (1-tension)/double(2);
	double u2, u3, ret;
	u2 = u * u;
	u3 = u2 * u;
	ret = p0 * (2*s*u2 - s*u3 - s*u)
		+ p1 * ((2-s)*u3 + (s-3)*u2 + 1)
		+ p2 * ((s-2)*u3 + (3-2*s)*u2 + s*u)
		+ p3 * (s*u3 - s*u2);
	return ret;
}

