// This codes was borrow from Professor Michael Gleicher

// $Header: /home2/cvsroot/proj2/RunButton.cpp,v 1.7 2000/04/19 01:09:14 cvs Exp $
//
// CS638 Example code - this provides a utility routine to help with animation
// written October, 1999 by Michael L. Gleicher
//
// This file implements a "play" button. The idea is that you creat a window that
// does your drawing. When redrawing, this window looks at a slider to see what
// time it is, so it knows what part of the animation to draw.
//
// The slider must call the window's redraw function whenever the time changes.
//
// The play button, when pressed, advances the slider forward a timestep continually.
// In order to do this, it installs itself in the FlTk "Idle" loop so it gets called
// periodically.
//

#include "RunButton.h"
#include <time.h>
#include <stdio.h>

// the guts of the slider - 
void runSliderCB(Fl_Widget*, void* )
{
	Fl_Widget *w;
	for(int i=0; i<wtuV.size(); i++)
	{
		w = wtuV[i];
		w->damage(1);
	}
}

// the slider constructor
RunSlider::RunSlider(Fl_Widget* dr, int length,
					 int x, int y, int w, int h) : Fl_Value_Slider(x,y,w,h)
{
	// set up slider parameters
	value(1);
	range(1,length);
	step(1);
	type(5);				// horizontal, "nice"
	when(FL_WHEN_CHANGED);	// update whenever it changes

	wtuV.push_back( dr );

	callback((Fl_Callback*) runSliderCB );
}

void RunSlider::RunLength(int length)
{
	range (1, length) ;
}

// Add widget to the damage list of this slider if its not already on there
void RunSlider::addWidgetToDamageList( Fl_Widget* w )
{
	for(int i=0; i<wtuV.size(); i++)
	{
		if( wtuV[i] == w ){
			return;
		}
	}
	wtuV.push_back( w );
}

// Remove widget from damage list of this slider if it exists
void RunSlider::removeWidgetFromDamageList( Fl_Widget* w )
{
	widgetsToUpdateVector::iterator wtuVpos = wtuV.begin();
	for(int i=0; i<wtuV.size(); i++)
	{
		if( wtuV[i] == w ){
			wtuV.erase( wtuVpos );
			return;
		}
		wtuVpos++;
	}
}



//
// this callback function is the "guts" of the RunButton -
// notice that it is an "idle" callback, not a widget callback
//
// another nice problem to have - most likely, we'll be too fast
// don't draw more than 30 times per second
void runButtonCB(RunButton* b)
{
	if (b->value()) {	// only advance time if appropriate
		if (clock() - b->lastRedraw > CLOCKS_PER_SEC*b->frameTime) {
			b->lastRedraw = (float)clock();
			double t = b->slider->value();
			double m = b->slider->maximum();
			t = t + b->slider->step();
			if (t>=m) t=1;	// wrap around
			b->slider->value(t);
			// note: FlTk doesn't have the slider call its callback
			// when the value is changed internally
			(b->slider->callback())(b->slider,b->slider->user_data());
		}
	}
}

// create the run button - the main thing it does is install the idle
// handler
RunButton::RunButton(Fl_Slider* time, 
					 int x, int y, int w, int h) : Fl_Light_Button(x,y,w,h,"Play")
{
	frameTime = 0.033333f;
	Fl::add_idle((void (*)(void*))runButtonCB,this);
	slider = time;
	lastRedraw = 0;
}
RunButton::~RunButton()
{
	Fl::remove_idle((void (*)(void*))runButtonCB,this);
}

void RunButton::setFrameTime(float ft)
{
	frameTime = ft;
}
