#ifndef _ANIM_WINDOW_H
#define _ANIM_WINDOW_H

/********************************************************************************
*
*	Files: animWindow.h and animWindow.cpp
*	Written by Steve Dutcher and Andy Gardner
*	
*	This class is the glWindow which draws the animation.  Quaternion camera
*	motion has been implemented in here.  Another thing to note is that the 
*	one important part of this class is that the user can get then data it needs
*	from this class in the whoIsShown array and then pass that into the constructor
*	for creating a clone of the animation window so that the user can see two
*	views at the same time
*
**********************************************************************************/

#include "fltkIncludes.h"
#include "animWindow.h"
#include "RunButton.h"

#include "motionData.h"

enum SHOWN_DATA_TYPES { ORIGINAL_MARKER=0, ORIGINAL_LINE, ORIGINAL_ELLIPSOID, EULER_MARKER, EULER_LINE, EULER_ELLIPSOID, QUATERNION_MARKER, QUATERNION_LINE, QUATERNION_ELLIPSOID };

class animWindow : public Fl_Gl_Window {
public:

	// This is the constructor for the GL_Window.  This will be called in the main function
	animWindow(int x, int y, int w, int h, motionData *md, int* shownValues=NULL);
	~animWindow();

	// These functions are called to initialize the animWindow when new files are loaded by the UI.
	void openMotionDataFile( char* );
	void openAcclaimDataFiles( char*, char* ) ;

	Fl_Slider *time;				// This pointer will be set in main to the slider controlling animation
	Fl_Button *play;				// This pointer will be set in main to the button to start playing

	void attachToSlider();			// These functions lock and dislocate the window values
	void detachFromSlider();		// to and from the time slider.
	
	void setShown(int, int);

private:

	//  draw all of the motionData as told to by the array whoIsShown
	void drawMotionData(int shadow=0);

	// draw individual joint line for a joint
	void drawJointLine(joint*, int, int shadow=0);

	// draw ellipsoid for a certain joint
	void drawJointEllipsoid(joint*, int, int shadow=0);

	// draw markers with trails for a joint
	void drawMarkers(int, int shadow=0);

	// draws the floor.  this gets passed a variable floor height so that it automatically
	// set the floor height to the lowest level that the character reaches
	void drawFloor( float floorHeight=0 );

	void draw();					// standard FlTk
	int handle(int);				// standard FlTk


	// store the viewing parameters - these may get changed as the user
	// moves things around

	double rotateCX, rotateCY, scaleC, translateCX, translateCY, translateCZ ;
	double rotateCXOld, rotateCYOld, scaleCOld ;
	double fieldOfView;
	float lookAtX, lookAtY, lookAtZ ;
	float lookAtXOld, lookAtYOld, lookAtZOld ;
	float translateX, translateZ ;

	int frame;

	// This is an array of which values are shown.  It should be used in conjunction
	// with the SHOWN_DATA_TYPES.  So you would say whoIsShon[EULER_MARKER] = 1 or 0
	int whoIsShown[9];

	motionData* mdata;
};

#endif
