#ifndef _GRAPH_EDITOR_H_
#define _GRAPH_EDITOR_H_

#pragma warning(disable: 4786)
#pragma warning(disable: 4788)

#include "fltkIncludes.h"
#include "motionData.h"

#include "jointBrowser.h"

// The pVector is intended to be a vector of points.
// These points are arranged in this Standard, Static fashion, and must be adhered
//   to accordingly.  <RX RY RZ TX TY TZ...>
// Subsequent internal (ONLY) references to the point vectors take this into consideration.
typedef std::vector<float> pVector   ;

// A large vector containing multiple point vectors.  This is used for multiple line-selection.
typedef std::vector<pVector> lVector ;

// A vector of selection masks.  If a point is selected, the integral value is 1, otherwise it
// is assigned a 0.
typedef std::vector<int> maskVector  ;

// This is the gl window aka the graph
class Gl_Graph_Window : public Fl_Gl_Window {

private:
	int handle(int);
	void draw();					// Simply draw the window - fltk fn.

	Fl_Slider  *time;				// This pointer will be set in main to the slider controlling animation
	Fl_Browser *jointBrowser;

	void frameAll() ;				// Frames all data in the graph editor proper.

	motionData *mdata ;				// The pointer to the data that is being gathered.

	int numberOfChannels ;			// How many channels are currently displayed.
	lVector lineVector   ;			// A vector of pointVectors.  See above...

	maskVector isEffector ;			// A vector of what is and/or is not an effector
									// this is used to not draw rotation curves for effectors.

	// This function is internally called when the mouse button is released.  It takes all points
	// located in the range specified by the mouse drag and initializes all of the required data
	// structures.  The offset value is 0 for X, 1 for Y, and 2 for Z.
	void lineSelectionProcess(pVector& tempPointVec, pVector& tempSelMask, int offset) ;

	// This function manipulates the point in the window so that they appear to be dragging
	// around.
	void dragPoints(int yDisplace) ;

	// This function is what actually displaces the points' data after the dragging operation
	// is complete.
	void doneDraggingPoints(void)  ;

	// The extents to which the window is visible.
	float left, right, top, bottom;

	// The largest value any curve keyframe in the window has.
	float topMax, bottomMax ;

	// Where the user clicked and/or dragged in the window.
	float clickX, clickY, dragX, dragY ;
	// Is a box currently being drawn?
	bool boxFlag ;
	// Is something currently selected?
	bool selectionFlag ;
	// A large space-delimited list of all of the joints currently active in the graph editor.
	char * bigNameString ;
	// The number of node names currently active in the graph editor.
	int numNodeNames     ;
	// Which of the six curves for any given instance of the graph editor is being displayed.
	int whoIsShown[6];

public:
	// Used to tell the graph editor window which curves to draw.
	void setShown(int,int);

	Gl_Graph_Window(int, int, int, int, motionData*, Fl_Slider*, Fl_Browser*) ;
	~Gl_Graph_Window();

	// Used to tell the graph editor, via a space-delimited selection list, the number of
	// names in that list, and whether or not the call is done internally or externally,
	// what data to display and where.
	void initGraphEditor(char * selection, int numNames, int dragFlag) ;
	// Interpolates between the selected points.  Takes no arguments, as it solely looks
	// through the selection list and interpolates what is needed.
	void interpBetweenSelected(void) ;
};




// This is the entire fltk window located on the screen.
class Graph_Editor_Window : public Fl_Window {
	Fl_Group *group;

	// Set up the proper buttons for the window.
	Fl_Light_Button rxButton, ryButton, rzButton;
	Fl_Light_Button txButton, tyButton, tzButton;
	Fl_Button interpButton;

	Fl_Button exitButton;
	Gl_Graph_Window graphWindow;
	JointBrowser jBrowser;
private:
	motionData *mdata;

public:
	Graph_Editor_Window(char*, motionData*, Fl_Slider*);
	void Graph_Editor_Window_CB(int);
};



#endif
