#include <stdio.h>
#include "jointBrowser.h"

/**************************************************************************
*
*	Files: jointBrowser.h and jointBrowser.cpp
*	Written by Steve Dutcher and Andy Gardner
*
*	This is just an fltk multi browser window that contains a list of
*	joint names.  When the user selects a name or names from this list
*	this window will set the variable haveChanged and the user can then
*	ask in its draw function has anything changed.  If there has been a 
*	change the user can then ask for the selected names in the list
*
*	NOTE:  read the file jointBrowser.h for descriptions of what functions do
*
**************************************************************************/

JointBrowser::JointBrowser(int x, int y, int w, int h, motionData* md, Fl_Widget* parent, char *label) :
	  Fl_Multi_Browser(x, y, w, h, label)
{
	mdata = md;
	myParent = parent ;

	// This loop will go through adding joints to the list 
	// in a heirarchial fashio
	for(int i=0; i<mdata->getNumberOfRoots(); i++)
	{
		addJointAndChildren( mdata->getRootName( i ) );
	}

	// This loop puts the names in the browser box
	for(int k=0; k<jNames.size(); k++)
	{
		add( jNames[k], (void*)k );
	}

	// Set callback so that GL_Window gets damaged on selection
	callback((Fl_Callback*) damageCB, myParent);
}

// Function intended to simply add the current joint and its children to the browser.
// It uses depth-first search to discover all of the joints.
void JointBrowser::addJointAndChildren( char *name )
{
	jNames.push_back( strdup(name) );
	
	joint *j = mdata->getJoint( name );
	for(int i=0; i<j->numOfChildren(); i++)
	{	
		if( j->getType() != EFFECTOR )
			addJointAndChildren( j->getChild( i ) );
	}
}

// Returns whether or not a joint has changed.
int JointBrowser::didIChange()
{
	return haveChanged ;
}

// Only used internally to the jointBrowser class.
void JointBrowser::iChanged()
{
	haveChanged = 1;
}

// Returns by 'reference' a space-delimited list of names for use with windows
// which would like such a thing.
int JointBrowser::getSelectedNames( char* names )
{
	int charArraySize = 0;
	int numOfNames = 0;
	for(int i=1; i<=size(); i++)
	{
		if( selected(i) )
			charArraySize += strlen( jNames[i-1] ) + 1;
	}
	if( charArraySize <= 0 )
		return 0;

	names[0] = '\0';

	for(int k=1; k<=size(); k++)
	{
		if( selected(k) )
		{
			names = strcat(names, jNames[k-1]);
			names = strcat(names, " ");
			numOfNames++;
		}
	}
	haveChanged = 0;
//	printf("names = %s\n",names);
	return numOfNames;
}

void damageCB(Fl_Widget* myself, Fl_Widget* o)
{
	((JointBrowser*)myself)->iChanged();
	o->damage(1);
}
