#ifndef _JOINT_BROWSER_H_
#define _JOINT_BROWSER_H_

/**************************************************************************
*
*	Files: jointBrowser.h and jointBrowser.cpp
*	Written by Steve Dutcher and Andy Gardner
*
*	This is just an fltk multi browser window that contains a list of
*	joint names.  When the user selects a name or names from this list
*	this window will set the variable haveChanged and the user can then
*	ask in its draw function has anything changed.  If there has been a 
*	change the user can then ask for the selected names in the list
*
**************************************************************************/

#pragma warning(disable: 4786)
#pragma warning(disable: 4788)

#include "fltkIncludes.h"
#include "motionData.h"
#include "joint.h"

#include <vector>

typedef std::vector<char*> jointNamesVector;

void damageCB(Fl_Widget*, Fl_Widget* o);

// This is the Browser Window for joint names
class JointBrowser : public Fl_Multi_Browser {

private:
	motionData *mdata;
	jointNamesVector jNames;
	int haveChanged;
	Fl_Widget* myParent;

	// function intended to simply add the current joint and its children to the browser.
	void addJointAndChildren( char *name );

public:

	// parent who is passed in here should be a pointer to the window who wants
	// to get updated so that I can call damage on you when I changed.  Then you
	// can check for my changes in your draw function
	JointBrowser(int x, int y, int w, int h, motionData* md, Fl_Widget* parent, char *label=0);

	// If there was a change I call this function.  Users should never call this
	void iChanged();

	// You can ask me if there was a change with this function
	int didIChange();

	// Get a space delimited list of names, you are responsible for freeing the memory that
	// I give you from this function.  I also return an int which is the nubmer of names
	// in the string
	int getSelectedNames( char* returnNames );
};

#endif