#ifndef _MAIN_WINDOW_H
#define _MAIN_WINDOW_H

/**************************************************************************************************
*
*	Files: mainWindow.h and mainWindow.cpp
*	Written by Steve Dutcher and Andy Gardner
*
*	This class handles the main window which has the buttons for what is displayed,
*	the time slider, menubar and animation window.  The user from this window can
*	also create a graphEditor window along with tearing off a copy of the animation
*	window, that will keep playing while open.
*
**************************************************************************************************/

#include "RunButton.h"
#include "animWindow.h"
#include "motionData.h"
#include "fltkIncludes.h"

#include <Fl/Fl_Output.H>

class mainWindow : public Fl_Double_Window {
	Fl_Group group;
	Fl_Menu_Bar menubar;

	// These are the buttons for showing data for ORIGINAL, EULER and QUATERNION
	// First letter is either O, E, or U then either
	// m for Marker jl for jointLine or je for jointEllipsoid
	Fl_Light_Button omButton, ojlButton, ojeButton;
	Fl_Light_Button emButton, ejlButton, ejeButton;
	Fl_Light_Button qmButton, qjlButton, qjeButton;

	Fl_Group text1, text2, text3, text4;

public:
	RunButton *rb;	// This is the play button
	RunSlider *rs;	// This is the time slider

	mainWindow(int, int, int, int, char*, motionData*);		// You must give this a pointer to the motionData
	~mainWindow();
	
	void cloneAnimationWindow();	// Create a copy of the animationWindow

private:
	int close;
	animWindow *aw;
	motionData *mdata;
};

#endif