#include <Fl/Enumerations.h>

#include "windowManager.h"
#include <stdio.h>

static mainWindow *mw;

enum box_type{ ONEPANEL, SIDEBYSIDE, TOPBOTTOM, SPLITLEFT, SPLITRIGHT, FOURPANELS };

void layout_cb( Fl_Widget*, void* option){
	mw->setLayout( (int)option );
}
void open_bvh_cb( Fl_Widget*, void* option){
	mw->openBvhFile();
}

mainWindow::mainWindow( int x, int y, int w, int h, char* winName, motionData *md) :
	Fl_Double_Window( x, y, w, h, winName ),
	menubar( 0, 0, w, 30 ),
	boxGroup( x, y, w-30, h-30 )
{
	mw = this;
	mdata = md;

	rs = new RunSlider( 0, 60, h-30, w-60, 30);
	rb = new RunButton( rs, 0, h-30, 60, 30);

	box1 = new defaultBox(0,30,w,h-60, mdata,rs,1);
	box2 = new defaultBox(0,30,w,h-60, mdata,rs,1);
	box4 = new defaultBox(0,30,w,h-60, mdata,rs);
	box3 = new defaultBox(0,30,w,h-60, mdata,rs);
	box1->box(FL_BORDER_BOX);
	box2->box(FL_BORDER_BOX);
	box3->box(FL_BORDER_BOX);
	box4->box(FL_BORDER_BOX);
	
	boxGroup.add( box1 );
	boxGroup.add( box2 );
	boxGroup.add( box3 );
	boxGroup.add( box4 );
	boxGroup.add( rs );
	boxGroup.add( rb );
		
	resizable( boxGroup );

	menubar.add("File/Open BVH File",0,open_bvh_cb, 0);	
	menubar.add("Display/Layout/One Panel",0,layout_cb, (void*)ONEPANEL);
	menubar.add("Display/Layout/Side by Side",0,layout_cb,(void*)SIDEBYSIDE);
	menubar.add("Display/Layout/Top and Bottom",0,layout_cb,(void*)TOPBOTTOM);
	menubar.add("Display/Layout/Split Left",0,layout_cb,(void*)SPLITLEFT);
	menubar.add("Display/Layout/Split Right",0,layout_cb,(void*)SPLITRIGHT);
	menubar.add("Display/Layout/Four Panels",0,layout_cb,(void*)FOURPANELS);
	damage(FL_DAMAGE_ALL);	

	setLayout(ONEPANEL);

	rs->addWidgetToDamageList( this );
}

mainWindow::~mainWindow()
{
	mw=NULL;
}
void mainWindow::setLayout( int option )
{
	if( option == ONEPANEL ){
		box1->resize(0,30,w(),h()-60);
	}else if( option == SIDEBYSIDE ){
		box1->resize(w()/2, 30, w()/2, h()-60);
		box2->resize(0,30,w()/2,h()-60);
	}else if( option == TOPBOTTOM ){
		box1->resize(0, 30, w(), h()/2-30);
		box2->resize(0, h()/2, w(), h()/2-30);
	}else if( option == SPLITRIGHT ){
		box1->resize(w()/3, 30, w()-w()/3, h()/2-30);
		box2->resize(0, 30, w()/3, h()-60);
		box3->resize(w()/3, h()/2, w()-w()/3, h()/2-30);
	}else if( option == SPLITLEFT ){
		box1->resize(0, 30, w()-w()/3, h()/2-30);
		box2->resize(0, h()/2, w()-w()/3, h()/2-30);
		box3->resize(2*w()/3, 30, w()/3, h()-60);
	}else if( option == FOURPANELS ){
		box1->resize(0, 30, w()-w()/2, h()/2-30);
		box2->resize(0, h()/2, w()-w()/2, h()/2-30);
		box3->resize(w()/2, 30, w()-w()/2, h()/2-30);
		box4->resize(w()/2, h()/2, w()-w()/2, h()/2-30);
	}

	if( option >= FOURPANELS ){
		box1->show();
		box2->show();
		box3->show();
		box4->show();
	}else if( option >= SPLITLEFT ){
		box1->show();
		box2->show();
		box3->show();
		box4->hide();
	}else if( option >= SIDEBYSIDE ){
		box1->show();
		box2->show();
		box3->hide();
		box4->hide();
	}else{
		box1->show();
		box2->hide();
		box3->hide();
		box4->hide();
	}

	damage(FL_DAMAGE_ALL);	
}

void mainWindow::openBvhFile()
{
	char *filename = fl_file_chooser("Choose a BVH file to open","*.bvh",NULL);
	if(filename == NULL || filename[0] == '\0')
		return;

	mdata->clearMotionData();
	mdata->readBVHfile( filename );

	rs->RunLength( mdata->getNumberOfFrames() ) ;
	rs->value(1);
	rs->damage(1);
}


#ifdef _MAIN

int main(int argc, char **argv)
{
	motionData mdata;

	mainWindow mw(30, 30, 600, 600, "Motion Data Program", &mdata);
	
	mw.show();
	mw.end();

	return Fl::run();
}

#endif
